<?php

declare(strict_types=1);

namespace App\Controller\OgAgent;

use App\Dto\Input\PartitionPostInput;
use App\Entity\Client;
use App\Entity\Command;
use App\Entity\Image;
use App\Entity\Trace;
use App\Model\ClientStatus;
use App\Model\CommandTypes;
use App\Model\ImageStatus;
use App\Model\TraceStatus;
use App\Service\Trace\CreateService;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class PartitionAssistantAction extends AbstractController
{
    public function __construct(
        protected readonly EntityManagerInterface $entityManager,
        protected readonly HttpClientInterface    $httpClient,
        protected readonly CreateService          $createService,
    )
    {
    }

    public function __invoke(PartitionPostInput $input): JsonResponse
    {
        $partitions = $input->partitions;

        if (empty($partitions)) {
            throw new ValidatorException('Partitions is required');
        }

        /** @var Client $client */
        $client = $input->partitions[0]->client->getEntity();

        $data = [];
        $diskNumber = 0;
        $cacheSize = 0;
        $disks = [];
        $cpt = '';

        $data = [];
        $diskData = [];

        foreach ($partitions as $partition) {
            if ($partition->filesystem === 'CACHE') {
                $cacheSize = $partition->size * 1024;
                $disks[$partition->diskNumber] = $cacheSize;
            }

            $diskNumber = $partition->diskNumber;

            $data[] = [
                'par' => (string) $partition->partitionNumber,
                'cpt' => $partition->partitionCode,
                'sfi' => $partition->filesystem,
                'tam' => (string) ($partition->size * 1024),
                'ope' => $partition->format ? "1" : "0",
            ];
        }

        foreach ($disks as $diskNumber => $size) {
            $diskData[] = [
                'dis' => (string) $diskNumber,
                'che' => "0",
                'tch' => (string) $size,
            ];
        }

        $data = array_merge($diskData, $data);

        $result = [
            "nfn" => "Configurar",
            "dsk" => "1",
            "cfg" => $data,
            "ids" => "0"
        ];

        try {
            $response = $this->httpClient->request('POST', 'https://'.$client->getIp().':8000/CloningEngine/Configurar', [
                'verify_peer' => false,
                'verify_host' => false,
                'headers' => [
                    'Content-Type' => 'application/json',
                ],
                'json' => $result,
            ]);

        } catch (TransportExceptionInterface $e) {
            return new JsonResponse(
                data: ['error' => $e->getMessage()],
                status: Response::HTTP_INTERNAL_SERVER_ERROR
            );
        }

        $jobId = json_decode($response->getContent(), true)['job_id'];

        $client->setStatus(ClientStatus::BUSY);
        $this->entityManager->persist($client);
        $this->entityManager->flush();

        $this->createService->__invoke($client, CommandTypes::PARTITION_AND_FORMAT, TraceStatus::IN_PROGRESS, $jobId, []);

        return new JsonResponse(data: $client, status: Response::HTTP_OK);
    }
}
